from onnx_pytorch.code_gen_templates.temp_formatter import TempFormatter
from onnx_pytorch.code_trans import CodeTrans


class ConvCodeTrans(CodeTrans):

  def __init__(self, schema):
    super(ConvCodeTrans, self).__init__(schema)

  def get_torch_input_num(self):
    return 1

  def get_nn_name_str(self):
    return "\"Conv{}d\".format(len(value_infos[node.input[0]].type.tensor_type.shape.dim) - 2)"

  def init_str(self):
    init_str = f'''
    \'\'\'self.{{node_name}} = nn.{{nn_name}}(**{{params}})
    self.{{node_name}}.weight.data = self.__variables[\"{{weight_name}}\"]
    if {{bias}}:
      self.{{node_name}}.bias.data = self.__variables[\"{{bias_name}}\"]
\'\'\'.format(
    nn_name={self.get_nn_name_str()},
    weight_name=node.input[1],
    bias_name="Undefined" if len(node.input) == 2 else node.input[2],
    node_name=node.name,
    bias=len(node.input) > 2,
    params=params_str,)'''
    return init_str

  def trans_input(self):
    pass

  def trans_attr(self):
    trans = '{{pytorch_attr}}=attr_value_dict[\"{{onnx_attr}}\"]'
    self.add_attr(
        "dilations", "dilation",
        (trans +
         '[:].__repr__() if \"{curr_attr}\" in attr_value_dict else 1').format(
             curr_attr="dilations"))
    self.add_attr(
        "strides", "stride",
        (trans +
         '[:].__repr__() if \"{curr_attr}\" in attr_value_dict else 1').format(
             curr_attr="strides"))
    self.add_attr(
        "kernel_shape", "kernel_size",
        "{pytorch_attr}=onnx.numpy_helper.to_array(initializers[node.input[1]]).shape[2:].__repr__()"
    )
    self.add_attr("group", "groups")
    self.add_attr(
        "pads", "padding",
        (trans +
         '[:].__repr__() if \"{curr_attr}\" in attr_value_dict else 0').format(
             curr_attr="pads"))
    self.add_attr(
        "kernel_shape", "in_channels",
        "{pytorch_attr}=onnx.numpy_helper.to_array(initializers[node.input[1]]).shape[1]"
    )
    self.add_attr(
        "kernel_shape", "out_channels",
        "{pytorch_attr}=onnx.numpy_helper.to_array(initializers[node.input[1]]).shape[0]"
    )



import glob
import inspect
import os
import torch.nn as nn

from onnx_pytorch.code_gen_templates.temp_formatter import TempFormatter

modules = glob.glob(os.path.join(os.path.dirname(__file__), "*.py"))
__all__ = [
    os.path.basename(f)[:-3]
    for f in modules
    if os.path.isfile(f) and not f.endswith('__init__.py')
] + ["get_op_trans"]

nn_dict = {
    nn_name.lower(): getattr(nn, nn_name)
    for nn_name in dir(nn)
    if nn_name[0].isupper()
}

nn_name_dict = {
    nn_name.lower(): nn_name for nn_name in dir(nn) if nn_name[0].isupper()
}


class CodeTrans:

  def __init__(self, schema=None):
    self.mapped_attrs = set()
    self.schema = schema

  def get_torch_input_num(self):
    curr_nn = nn_dict.get(self.schema.name.lower())
    if not curr_nn:
      return 1
    return len(inspect.getfullargspec(curr_nn.forward).args) - 1

  def get_nn_name_str(self):
    if not self.schema:
      return "Undefined"
    if self.schema.name.lower() in nn_name_dict:
      return "\"{}\"".format(nn_name_dict[self.schema.name.lower()])
    return "\"{}\"".format(
        nn_name_dict.get(self.schema.name.lower(), self.schema.name.lower()))

  def init_str(self):
    return TempFormatter.init_str(self.get_nn_name_str())

  def forward_str(self):
    return TempFormatter.forward_str()

  def additional_str(self):
    return ""

  def trans_input(self):
    pass

  def trans_attr(self):
    pass

  def add_attr(self, onnx_attr, pytorch_attr, trans=None):
    if trans is None:
      trans = TempFormatter.default_attr_trans(pytorch_attr=pytorch_attr,
                                               onnx_attr=onnx_attr)
    self.mapped_attrs.add((onnx_attr, pytorch_attr,
                           trans.format(onnx_attr=onnx_attr,
                                        pytorch_attr=pytorch_attr)))


def get_op_trans(op=None):
  mod = globals().get(op, None)
  if mod is None:
    return CodeTrans
  return getattr(mod, "{}CodeTrans".format(op))



from onnx_pytorch.code_gen_templates.temp_formatter import TempFormatter
from onnx_pytorch.code_trans import CodeTrans


class MaxPoolCodeTrans(CodeTrans):

  def __init__(self, schema):
    super(MaxPoolCodeTrans, self).__init__(schema)

  def get_nn_name_str(self):
    return "\"MaxPool{}d\".format(len(value_infos[node.input[0]].type.tensor_type.shape.dim) - 2)"

  def trans_attr(self):
    trans = '{{pytorch_attr}}=attr_value_dict[\"{{onnx_attr}}\"]'
    self.add_attr(
        "dilations", "dilation",
        (trans +
         '[:].__repr__() if \"{curr_attr}\" in attr_value_dict else 1').format(
             curr_attr="dilations"))
    self.add_attr(
        "strides", "stride",
        (trans +
         '[:].__repr__() if \"{curr_attr}\" in attr_value_dict else 1').format(
             curr_attr="strides"))
    self.add_attr("kernel_shape", "kernel_size",
                  (trans + '[:].__repr__()').format(curr_attr="kernel_shape"))
    self.add_attr(
        "pads", "padding",
        (trans +
         '[2:].__repr__() if \"{curr_attr}\" in attr_value_dict else 0').format(
             curr_attr="pads"))
    self.add_attr("ceil_mode", "ceil_mode",
                  "{pytorch_attr}=bool(attr_value_dict[\"ceil_mode\"])")
    self.add_attr("", "return_indices",
                  "{pytorch_attr}=(len(node.output) == 2)")



from onnx_pytorch.code_trans import CodeTrans


class AddCodeTrans(CodeTrans):

  def __init__(self, schema):
    super(AddCodeTrans, self).__init__(schema)

  def init_str(self):
    return "\"\""

  def forward_str(self):
    return f'''\'\'\'{{outputs_str}} = {{inputs_str}}\'\'\'.format(
            node_name=node.name,
            inputs_str=" + ".join(inputs_str),
            outputs_str=", ".join(outputs_str),)'''

  def additional_str(self):
    return '''inputs_str = ["self.{}".format(i) if i not in initializers else "self.__variables[\\"{}\\"]".format(i) for i in node.input]'''



from onnx_pytorch.code_gen_templates.temp_formatter import TempFormatter
from onnx_pytorch.code_trans import CodeTrans


class GlobalAveragePoolCodeTrans(CodeTrans):

  def __init__(self, schema):
    super(GlobalAveragePoolCodeTrans, self).__init__(schema)

  def get_nn_name_str(self):
    return "\"functional.avg_pool{}d\".format(len(value_infos[node.input[0]].type.tensor_type.shape.dim) - 2)"

  def init_str(self):
    return f'''"self.{{node_name}} = nn.{{nn_name}}".format(
    nn_name={self.get_nn_name_str()},
    node_name=node.name,)'''

  def trans_attr(self):
    self.add_attr(
        "", "kernel_size",
        "{pytorch_attr}=\"self.{{}}.shape[{{}}:]\".format(node.input[0], (len(value_infos[node.input[0]].type.tensor_type.shape.dim) - 2))"
    )

  def forward_str(self):
    return TempFormatter.forward_str_functional()



from onnx_pytorch.code_trans import CodeTrans


class FlattenCodeTrans(CodeTrans):

  def __init__(self, schema):
    super(FlattenCodeTrans, self).__init__(schema)

  def trans_attr(self):
    self.add_attr("axis", "start_dim")



import onnx
import onnxruntime
import torch
from onnx.helper import make_node, make_tensor_value_info, make_graph, make_model, make_tensor
import numpy as np
from onnxruntime.tools.symbolic_shape_infer import SymbolicShapeInference

from onnx_pytorch import OnnxModel


def conv_flatten_relu():
  t1 = make_tensor("Conv_input_1", onnx.TensorProto.FLOAT, [32, 3, 3, 3],
                   np.random.randn(32, 3, 3, 3).astype(np.float32).flatten())
  t2 = make_tensor("Conv_input_2", onnx.TensorProto.FLOAT, [32],
                   np.random.randn(32).astype(np.float32).flatten())
  n = make_node(
      "Conv",
      ["Conv_input_{}".format(i) for i in range(3)],
      ["Conv_output_{}".format(i) for i in range(1)],
  )
  n1 = make_node(
      "Flatten",
      n.output,
      ["Flatten_output_{}".format(i) for i in range(1)],
  )
  n2 = make_node(
      "Relu",
      n1.output,
      ["Relu_output_{}".format(i) for i in range(1)],
  )
  input_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, [1, 3, 224, 224])
      for i in n.input[:1]
  ]
  output_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, None) for i in n2.output
  ]
  model = make_model(
      make_graph([n, n1, n2], "", input_vis, output_vis, initializer=[t1, t2]))

  sess_options = onnxruntime.SessionOptions()
  session = onnxruntime.InferenceSession(model.SerializeToString(),
                                         sess_options)
  inputs = {"Conv_input_0": np.random.randn(1, 3, 224, 224).astype(np.float32)}
  ort_outputs = session.run(None, inputs)
  print(ort_outputs)

  model.graph.ClearField("value_info")
  model = SymbolicShapeInference.infer_shapes(model, 2**31 - 1, True, True, 1)
  onnx.save(model, "conv_flatten_relu.onnx")


def conv_flatten_add_relu():
  t1 = make_tensor("Conv_input_1", onnx.TensorProto.FLOAT, [32, 3, 3, 3],
                   np.random.randn(32, 3, 3, 3).astype(np.float32).flatten())
  t2 = make_tensor("Conv_input_2", onnx.TensorProto.FLOAT, [32],
                   np.random.randn(32).astype(np.float32).flatten())
  t3 = make_tensor("Add_input_1", onnx.TensorProto.FLOAT, [1, 1577088],
                   np.random.randn(1577088).astype(np.float32).flatten())
  n = make_node(
      "Conv",
      ["Conv_input_{}".format(i) for i in range(3)],
      ["Conv_output_{}".format(i) for i in range(1)],
  )
  n1 = make_node(
      "Flatten",
      n.output,
      ["Flatten_output_{}".format(i) for i in range(1)],
  )
  n2 = make_node(
      "Add",
      list(n1.output) + [t3.name],
      ["Add_output_{}".format(i) for i in range(1)],
  )
  n3 = make_node(
      "Relu",
      n2.output,
      ["Relu_output_{}".format(i) for i in range(1)],
  )
  input_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, [1, 3, 224, 224])
      for i in n.input[:1]
  ]
  output_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, None) for i in n3.output
  ]
  model = make_model(
      make_graph([n, n1, n2, n3],
                 "",
                 input_vis,
                 output_vis,
                 initializer=[t1, t2, t3]))
  sess_options = onnxruntime.SessionOptions()
  session = onnxruntime.InferenceSession(model.SerializeToString(),
                                         sess_options)
  inputs = {"Conv_input_0": np.random.randn(1, 3, 224, 224).astype(np.float32)}
  ort_outputs = session.run(None, inputs)
  print(ort_outputs)

  model.graph.ClearField("value_info")
  model = SymbolicShapeInference.infer_shapes(model, 2**31 - 1, True, True, 1)
  onnx.save(model, "conv_flatten_add_relu.onnx")


def conv_maxpool_flatten_add_relu():
  t1 = make_tensor("Conv_input_1", onnx.TensorProto.FLOAT, [32, 3, 3, 3],
                   np.random.randn(32, 3, 3, 3).astype(np.float32).flatten())
  t2 = make_tensor("Conv_input_2", onnx.TensorProto.FLOAT, [32],
                   np.random.randn(32).astype(np.float32).flatten())
  t3 = make_tensor("Add_input_1", onnx.TensorProto.FLOAT, [1],
                   np.random.randn(1).astype(np.float32).flatten())
  n = make_node(
      "Conv",
      ["Conv_input_{}".format(i) for i in range(3)],
      ["Conv_output_{}".format(i) for i in range(1)],
  )
  n1 = make_node(
      "MaxPool",
      n.output,
      ["MaxPool_output_{}".format(i) for i in range(1)],
      kernel_shape=(3, 3),
      strides=(2, 2),
      pads=(0, 0, 1, 1),
  )
  n2 = make_node(
      "Flatten",
      n1.output,
      ["Flatten_output_{}".format(i) for i in range(1)],
  )
  n3 = make_node(
      "Add",
      list(n2.output) + [t3.name],
      ["Add_output_{}".format(i) for i in range(1)],
  )
  n4 = make_node(
      "Relu",
      n3.output,
      ["Relu_output_{}".format(i) for i in range(1)],
  )
  input_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, [1, 3, 224, 224])
      for i in n.input[:1]
  ]
  output_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, None) for i in n4.output
  ]
  model = make_model(
      make_graph([n, n1, n2, n3, n4],
                 "",
                 input_vis,
                 output_vis,
                 initializer=[t1, t2, t3]))
  sess_options = onnxruntime.SessionOptions()
  session = onnxruntime.InferenceSession(model.SerializeToString(),
                                         sess_options)
  inputs = {"Conv_input_0": np.random.randn(1, 3, 224, 224).astype(np.float32)}
  ort_outputs = session.run(None, inputs)
  print(ort_outputs)

  model.graph.ClearField("value_info")
  model = SymbolicShapeInference.infer_shapes(model, 2**31 - 1, True, True, 1)
  onnx.save(model, "conv_maxpool_flatten_add_relu.onnx")


def conv_maxpool_globalavgpool_flatten_add_relu():
  t1 = make_tensor("Conv_input_1", onnx.TensorProto.FLOAT, [32, 3, 3, 3],
                   np.random.randn(32, 3, 3, 3).astype(np.float32).flatten())
  t2 = make_tensor("Conv_input_2", onnx.TensorProto.FLOAT, [32],
                   np.random.randn(32).astype(np.float32).flatten())
  t3 = make_tensor("Add_input_1", onnx.TensorProto.FLOAT, [1],
                   np.random.randn(1).astype(np.float32).flatten())
  n = make_node(
      "Conv",
      ["Conv_input_{}".format(i) for i in range(3)],
      ["Conv_output_{}".format(i) for i in range(1)],
  )
  n1 = make_node(
      "MaxPool",
      n.output,
      ["MaxPool_output_{}".format(i) for i in range(1)],
      kernel_shape=(3, 3),
      strides=(2, 2),
      pads=(0, 0, 1, 1),
  )
  n2 = make_node(
      "GlobalAveragePool",
      n1.output,
      ["GlobalAveragePool_output_{}".format(i) for i in range(1)],
  )
  n3 = make_node(
      "Flatten",
      n2.output,
      ["Flatten_output_{}".format(i) for i in range(1)],
  )
  n4 = make_node(
      "Add",
      list(n3.output) + [t3.name],
      ["Add_output_{}".format(i) for i in range(1)],
  )
  n5 = make_node(
      "Relu",
      n4.output,
      ["Relu_output_{}".format(i) for i in range(1)],
  )
  input_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, [1, 3, 224, 224])
      for i in n.input[:1]
  ]
  output_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, None) for i in n5.output
  ]
  model = make_model(
      make_graph([n, n1, n2, n3, n4, n5],
                 "",
                 input_vis,
                 output_vis,
                 initializer=[t1, t2, t3]))
  sess_options = onnxruntime.SessionOptions()
  session = onnxruntime.InferenceSession(model.SerializeToString(),
                                         sess_options)
  inputs = {"Conv_input_0": np.random.randn(1, 3, 224, 224).astype(np.float32)}
  ort_outputs = session.run(None, inputs)
  print(ort_outputs)

  model.graph.ClearField("value_info")
  model = SymbolicShapeInference.infer_shapes(model, 2**31 - 1, True, True, 1)
  onnx.save(model, "conv_maxpool_globalavgpool_flatten_add_relu.onnx")


def conv_batchnorm_maxpool_globalavgpool_flatten_add_relu():
  t1 = make_tensor("Conv_input_1", onnx.TensorProto.FLOAT, [32, 3, 3, 3],
                   np.random.randn(32, 3, 3, 3).astype(np.float32).flatten())
  t2 = make_tensor("Conv_input_2", onnx.TensorProto.FLOAT, [32],
                   np.random.randn(32).astype(np.float32).flatten())
  t3 = make_tensor("Add_input_1", onnx.TensorProto.FLOAT, [1],
                   np.random.randn(1).astype(np.float32).flatten())
  t4 = make_tensor("BatchNormalization_input_0", onnx.TensorProto.FLOAT, [32],
                   np.ones(32).astype(np.float32).flatten())
  t5 = make_tensor("BatchNormalization_input_1", onnx.TensorProto.FLOAT, [32],
                   np.zeros(32).astype(np.float32).flatten())
  t6 = make_tensor("BatchNormalization_input_2", onnx.TensorProto.FLOAT, [32],
                   np.random.randn(32).astype(np.float32).flatten())
  t7 = make_tensor("BatchNormalization_input_3", onnx.TensorProto.FLOAT, [32],
                   np.random.randn(32).astype(np.float32).flatten())
  n = make_node(
      "Conv",
      ["Conv_input_{}".format(i) for i in range(3)],
      ["Conv_output_{}".format(i) for i in range(1)],
  )
  n1 = make_node(
      "BatchNormalization",
      list(n.output) + [t4.name, t5.name, t6.name, t7.name],
      ["BatchNormalization_output_{}".format(i) for i in range(1)],
  )
  n2 = make_node(
      "MaxPool",
      n1.output,
      ["MaxPool_output_{}".format(i) for i in range(1)],
      kernel_shape=(3, 3),
      strides=(2, 2),
      pads=(0, 0, 1, 1),
  )
  n3 = make_node(
      "GlobalAveragePool",
      n2.output,
      ["GlobalAveragePool_output_{}".format(i) for i in range(1)],
  )
  n4 = make_node(
      "Flatten",
      n3.output,
      ["Flatten_output_{}".format(i) for i in range(1)],
  )
  n5 = make_node(
      "Add",
      list(n4.output) + [t3.name],
      ["Add_output_{}".format(i) for i in range(1)],
  )
  n6 = make_node(
      "Relu",
      n5.output,
      ["Relu_output_{}".format(i) for i in range(1)],
  )
  input_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, [1, 3, 224, 224])
      for i in n.input[:1]
  ]
  output_vis = [
      make_tensor_value_info(i, onnx.TensorProto.FLOAT, None) for i in n6.output
  ]
  model = make_model(
      make_graph([n, n1, n2, n3, n4, n5, n6],
                 "",
                 input_vis,
                 output_vis,
                 initializer=[t1, t2, t3, t4, t5, t6, t7]))
  sess_options = onnxruntime.SessionOptions()
  session = onnxruntime.InferenceSession(model.SerializeToString(),
                                         sess_options)
  inputs = {"Conv_input_0": np.random.randn(1, 3, 224, 224).astype(np.float32)}
  ort_outputs = session.run(None, inputs)
  print(ort_outputs)

  model.graph.ClearField("value_info")
  model = SymbolicShapeInference.infer_shapes(model, 2**31 - 1, True, True, 1)
  onnx.save(model, "conv_batchnorm_maxpool_globalavgpool_flatten_add_relu.onnx")


def resnet_18():
  model = onnx.load("ort_train_resnet18.onnx")
  model.graph.ClearField("value_info")
  for n in model.graph.node:
    inputs, outputs = [], []
    for i in n.input:
      if i.isnumeric():
        inputs.append(i)
      else:
        i.replace(".", "_")
        inputs.append(i)
    for i in n.output:
      if i.isnumeric():
        outputs.append(i)
      else:
        i.replace(".", "_")
        outputs.append(i)

    n.ClearField("input")
    n.input.extend(inputs)
    n.ClearField("output")
    n.output.extend(outputs)

  for i in model.graph.input:
    if i.name.isnumeric():
      i.name = "__t_{}".format(i.name)
    else:
      i.name = i.name.replace(".", "_")
  for i in model.graph.output:
    if i.name.isnumeric():
      i.name = "__t_{}".format(i.name)
    else:
      i.name = i.name.replace(".", "_")

    for i in model.graph.initializer:
      if i.name.isnumeric():
        i.name = "__t_{}".format(i.name)
      else:
        i.name = i.name.replace(".", "_")
  onnx.save(model, "ort_train_resnet18.onnx")
  model = SymbolicShapeInference.infer_shapes(model, 2**31 - 1, True, True, 1)

  onnx.save(model, "ort_train_resnet18.onnx")


if __name__ == '__main__':
  resnet_18()



import argparse
import logging
import os
import re
import shutil

import numpy as np
import onnx
from onnx.numpy_helper import to_array
from onnxruntime.tools.symbolic_shape_infer import SymbolicShapeInference
import torch
import torch.nn as nn

from onnx_pytorch.code_gen_template import CodeGenTemplate
from onnx_pytorch.op_code_generators import *

torch.set_printoptions(6)

nn_dict = {
    nn_name.lower(): getattr(nn, nn_name)
    for nn_name in dir(nn)
    if nn_name[0].isupper()
}

nn_name_dict = {
    nn_name.lower(): nn_name for nn_name in dir(nn) if nn_name[0].isupper()
}


class ModelCodeGenerator:

  def __init__(self, onnx_model_path=None, output_dir=None, overwrite=False):
    self.onnx_model_path = onnx_model_path
    assert os.path.exists(
        onnx_model_path), f"ONNX model {onnx_model_path} does not exist."
    assert os.path.isfile(onnx_model_path), f"{onnx_model_path} is not a file."

    self.output_dir = output_dir
    assert os.path.exists(
        output_dir
    ) and overwrite is not True, f"{output_dir} is not empty and overwrite is not True."
    assert os.path.isdir(output_dir), f"{output_dir} is not directory."
    if overwrite:
      shutil.rmtree(output_dir, ignore_errors=True)
      os.makedirs(output_dir)

    self.onnx_model = None

    self.init_parts = []
    self.forward_parts = []

  def add_init_part(self, m):
    if type(m) in (list, tuple, set):
      self.init_parts.extend(m)
    else:
      self.init_parts.append(m)

  def add_forward_part(self, m):
    if type(m) in (list, tuple, set):
      self.forward_parts.extend(m)
    else:
      self.forward_parts.append(m)

  def add_forward_return(self, outputs_value_infos):
    return_list = [f"self.{o.name}" for o in outputs_value_infos]
    self.forward_parts.append(f"return {', '.join(return_list)}")

  def add_forward_input(self, inputs_value_infos):
    return_list = [f"self.{i.name}" for i in inputs_value_infos]
    if len(inputs_value_infos) == 1:
      self.forward_parts.append(f"{return_list[0]}, = inputs")
    else:
      self.forward_parts.append(f"{', '.join(return_list)} = inputs")

  def gen_model_code(self):
    return CodeGenTemplate.model(model_init='''
    '''.join(self.init_parts),
                                 model_forward='''
    '''.join(self.forward_parts))

  def preprocess_onnx_model(self):
    model = onnx.load(self.onnx_model_path)
    model.graph.ClearField("value_info")
    for n in model.graph.node:
      inputs, outputs = [], []
      for ls, f in ((inputs, n.input), (outputs, n.output)):
        for i in f:
          if i.isnumeric():
            ls.append(f"__t_{i}")
          else:
            ls.append(re.sub("[:/.]", "_", i))
          if i != ls[-1]:
            logging.warning(f"Tensor name {i} is changed to {ls[-1]}.")

      n.ClearField("input")
      n.input.extend(inputs)
      n.ClearField("output")
      n.output.extend(outputs)

    for f in (model.graph.input, model.graph.output, model.graph.initializer):
      for i in f:
        if i.name.isnumeric():
          i.name = f"__t_{i.name}"
        else:
          i.name = re.sub("[:/.]", "_", i.name)

    model = SymbolicShapeInference.infer_shapes(model, 2**31 - 1, True, True, 0)
    onnx.save(model, os.path.join(self.output_dir, "tmp_processed.onnx"))
    self.onnx_model = model

  def run(self):
    self.preprocess_onnx_model()
    initializers = {i.name: i for i in self.onnx_model.graph.initializer}
    input_value_infos = {i.name: i for i in self.onnx_model.graph.input}
    output_value_infos = {i.name: i for i in self.onnx_model.graph.output}
    value_infos = {}
    value_infos.update(input_value_infos)
    value_infos.update(output_value_infos)
    value_infos.update({i.name: i for i in self.onnx_model.graph.value_info})

    self.add_forward_input(self.onnx_model.graph.input)
    for n in self.onnx_model.graph.node:
      op_code_gen = get_op_code_generator(n.op_type)
      assert op_code_gen, "OpCodeGenerator is unimplemented for {}.".format(
          n.op_type)
      gened = op_code_gen.gen(n, value_infos, initializers)
      self.add_init_part(gened["init"])
      self.add_forward_part(gened["forward"])
    self.add_forward_return(self.onnx_model.graph.output)

    gened_code = self.gen_model_code()
    print(gened_code)
    with open(os.path.join(self.output_dir, "model.py"), "w") as f:
      f.write(gened_code)
    shutil.rmtree(os.path.join(self.output_dir, "variables"),
                  ignore_errors=True)
    os.makedirs(os.path.join(self.output_dir, "variables"))
    for k, v in initializers.items():
      np.save(os.path.join(self.output_dir, "variables", f"{k}.npy"),
              to_array(v))


def gen(onnx_model_path, output_dir, overwrite=False):
  model_code_gen = ModelCodeGenerator(onnx_model_path=onnx_model_path,
                                      output_dir=output_dir,
                                      overwrite=overwrite)
  model_code_gen.run()


def main():
  debug = True
  parser = argparse.ArgumentParser()
  parser.add_argument("--onnx_model_path",
                      default=None,
                      type=str,
                      required=not debug,
                      help="The onnx model path.")

  parser.add_argument("--output_dir",
                      default=None,
                      type=str,
                      required=not debug,
                      help="The output dir")

  parser.add_argument("--overwrite",
                      default=False,
                      type=bool,
                      help="Should overwrite the output dir.")
  args = parser.parse_args()

  # model_code_gen = ModelCodeGenerator(
  #     onnx_model_path="test/ort_train_resnet18.onnx", output_dir="./")
  model_code_gen = ModelCodeGenerator(onnx_model_path=args.onnx_model_path,
                                      output_dir=args.output_dir,
                                      overwrite=args.overwrite)
  model_code_gen.run()


if __name__ == '__main__':
  main()



from onnx_pytorch.code_gen_templates.temp_formatter import TempFormatter
from onnx_pytorch.code_trans import CodeTrans


class GemmCodeTrans(CodeTrans):

  def __init__(self, schema):
    super(GemmCodeTrans, self).__init__(schema)

  def get_torch_input_num(self):
    return 1

  def get_nn_name_str(self):
    return "\"Conv{}d\".format(len(value_infos[node.input[0]].type.tensor_type.shape.dim) - 2)"

  def init_str(self):
    return "\"\""

  def trans_input(self):
    pass

  def forward_str(self):
    return '''\'\'\'
    torch.matmul({{inputs_str}}, {{outputs_str}})
    \'\'\'.format(        
            inputs_str=", ".join(inputs_str),
            outputs_str=", ".join(outputs_str),)'''

  def additional_str(self):
    return '''
    inputs_str[0] = "torch.transpose({}, 0, 1)".format(inputs_str[0]) if attr_value_dict["transA"] == 1 else inputs_str[0]
    inputs_str[1] = "torch.transpose({}, 0, 1)".format(inputs_str[1]) if attr_value_dict["transB"] == 1 else inputs_str[1]
'''


import collections
import inspect
import dis
import itertools
import os
import shutil
import sys

import onnx_pytorch
import numpy as np
import onnx
from onnx import defs, TensorProto
import onnxruntime
import torch
import torch.nn as nn
from onnx.helper import make_node, make_empty_tensor_value_info, make_graph, make_model, make_tensor_value_info
from onnx.numpy_helper import to_array

from onnx_pytorch.code_gen_templates.temp_formatter import TempFormatter
from onnx_pytorch.code_trans import *

torch.set_printoptions(6)

MATH_OP = {
    "LessOrEqual": "<=",
}

nn_dict = {
    nn_name.lower(): getattr(nn, nn_name)
    for nn_name in dir(nn)
    if nn_name[0].isupper()
}

nn_name_dict = {
    nn_name.lower(): nn_name for nn_name in dir(nn) if nn_name[0].isupper()
}

ignore_torch_attr = {"inplace"}
ignore_onnx_attr = {}


def try_run_ort(schema, attrs={}, inputs={}):
  n = make_node(schema.name,
                ["input_{}".format(i) for i in range(schema.max_input)],
                ["output_{}".format(i) for i in range(schema.max_output)],
                **attrs)
  input_vis = [
      make_tensor_value_info(i, TensorProto.FLOAT, None) for i in n.input
  ]
  output_vis = [
      make_tensor_value_info(i, TensorProto.FLOAT, None) for i in n.output
  ]
  model = make_model(make_graph([n], "", input_vis, output_vis))
  sess_options = onnxruntime.SessionOptions()
  session = onnxruntime.InferenceSession(model.SerializeToString(),
                                         sess_options)
  ort_outputs = session.run(None, inputs)
  return ort_outputs


def try_run_torch(schema, attrs={}, inputs=[]):
  nn_instance = nn_dict[schema.name.lower()](**attrs)
  return nn_instance(*[torch.from_numpy(i) for i in inputs])


def try_mapping_args_WIP(schema):
  if schema.name.lower() not in nn_dict:
    return None
  curr_nn = nn_dict[schema.name.lower()]
  full_arg_spec = inspect.getfullargspec(curr_nn.__init__)
  args = full_arg_spec.args[1:]
  args_perm = collections.OrderedDict()
  for a in args:
    if a in ignore_torch_attr:
      continue
    anno = full_arg_spec.annotations[a]
    if anno is bool:
      args_perm[a] = (True, False)
    elif anno is int:
      args_perm[a] = (1, 2, 4)
  torch_attr_pool = list(itertools.product(args_perm.values()))
  print(torch_attr_pool)

  ort_inputs = collections.OrderedDict()
  for idx, i in enumerate(schema.inputs):
    ort_inputs["input_{}".format(idx)] = np.random.randn(1,
                                                         10).astype(np.float32)

  print(schema.attributes)

  if full_arg_spec.defaults:
    # ort_outputs = try_run_ort(schema, attrs={}, inputs=ort_inputs)
    torch_attr_default = {a: d for a, d in zip(args, full_arg_spec.defaults)}
    torch_output = try_run_torch(schema,
                                 attrs=torch_attr_default,
                                 inputs=list(ort_inputs.values()))
  elif not torch_attr_pool:
    ort_outputs = try_run_ort(schema, attrs={}, inputs=ort_inputs)
    torch_output = try_run_torch(schema,
                                 attrs={},
                                 inputs=list(ort_inputs.values()))
    assert np.allclose(ort_outputs, torch_output)
    print(torch_output)


def gen_code_gen():
  shutil.rmtree(os.path.join(os.path.dirname(__file__), "op_code_generators"))
  os.mkdir(os.path.join(os.path.dirname(__file__), "op_code_generators"))
  with open(
      os.path.join(os.path.dirname(__file__), "op_code_generators", "__init__.py"),
      "w") as f:
    f.write(TempFormatter.code_generator())
  for schema in defs.get_all_schemas():
    if schema.domain in ("ai.onnx.preview.training", "ai.onnx.ml"):
      continue
    if schema.name not in ("Conv", "Relu", "Gemm", "Add", "Flatten",
                           "GlobalAveragePool", "MaxPool",
                           "BatchNormalization"):
      continue
    trans_cls = get_op_trans(schema.name)
    if trans_cls is not None:
      trans_cls_ins = trans_cls(schema)
    else:
      trans_cls = get_op_trans()
      trans_cls_ins = trans_cls()
    trans_cls_ins.trans_attr()
    mapped_attrs = trans_cls_ins.mapped_attrs
    attr_default = "{{{attr_default_contents}}}"
    attr_default_contents = []
    for a, i in schema.attributes.items():
      try:
        default_value = onnx.helper.get_attribute_value(i.default_value)
        attr_default_contents.append("\"{key}\": {value}".format(
            key=a, value=default_value))
      except:
        continue
    attr_default_str = attr_default.format(
        attr_default_contents=", ".join(attr_default_contents))

    params_content = ""
    format_contents = []
    if mapped_attrs:
      for onnx_attr, pytorch_attr, trans in mapped_attrs:
        format_contents.append(trans)
        params_content += TempFormatter.single_attr(pytorch_attr=pytorch_attr)
    params_str = f'"{{{{{params_content}}}}}"'
    params_str += TempFormatter.format(format_str=", ".join(format_contents))
    gened_op_code_generator = TempFormatter.op_code_generator(
        attr_default_str=attr_default_str,
        op=schema.name,
        onnx_ver=schema.since_version,
        torch_ver=torch.__version__,
        input_num=trans_cls_ins.get_torch_input_num(),
        params_str=params_str,
        init_str=trans_cls_ins.init_str(),
        forward_str=trans_cls_ins.forward_str(),
        additional_str=trans_cls_ins.additional_str(),
    )

    with open(
        os.path.join(os.path.dirname(__file__), "op_code_generators",
                     "{}.py".format(schema.name)), "w") as f:
      f.write(gened_op_code_generator)
    pass
  pass


def gen_code():
  # shutil.rmtree(os.path.join(os.path.dirname(__file__), "op_code_generators"))
  # os.mkdir(os.path.join(os.path.dirname(__file__), "op_code_generators"))
  with open(
      os.path.join(os.path.dirname(__file__), "op_code_generators", "__init__.py"),
      "w") as f:
    f.write(TempFormatter.code_generator())
  # for schema in defs.get_all_schemas():
  #   if schema.domain in ("ai.onnx.preview.training", "ai.onnx.ml"):
  #     continue
  #   if schema.name not in ("Conv", "Relu", "Gemm", "Add", "Flatten",
  #                          "GlobalAveragePool", "MaxPool",
  #                          "BatchNormalization"):
  #     continue
  #   trans_cls = get_op_trans(schema.name)
  #   if trans_cls is not None:
  #     trans_cls_ins = trans_cls(schema)
  #   else:
  #     trans_cls = get_op_trans()
  #     trans_cls_ins = trans_cls()
  #   trans_cls_ins.trans_attr()
  #   mapped_attrs = trans_cls_ins.mapped_attrs
  #   attr_default = "{{{attr_default_contents}}}"
  #   attr_default_contents = []
  #   for a, i in schema.attributes.items():
  #     try:
  #       default_value = onnx.helper.get_attribute_value(i.default_value)
  #       attr_default_contents.append("\"{key}\": {value}".format(
  #         key=a, value=default_value))
  #     except:
  #       continue
  #   attr_default_str = attr_default.format(
  #     attr_default_contents=", ".join(attr_default_contents))
  #
  #   params_content = ""
  #   format_contents = []
  #   if mapped_attrs:
  #     for onnx_attr, pytorch_attr, trans in mapped_attrs:
  #       format_contents.append(trans)
  #       params_content += TempFormatter.single_attr(pytorch_attr=pytorch_attr)
  #   params_str = f'"{{{{{params_content}}}}}"'
  #   params_str += TempFormatter.format(format_str=", ".join(format_contents))
  #   gened_op_code_generator = TempFormatter.op_code_generator(
  #     attr_default_str=attr_default_str,
  #     op=schema.name,
  #     onnx_ver=schema.since_version,
  #     torch_ver=torch.__version__,
  #     input_num=trans_cls_ins.get_torch_input_num(),
  #     params_str=params_str,
  #     init_str=trans_cls_ins.init_str(),
  #     forward_str=trans_cls_ins.forward_str(),
  #     additional_str=trans_cls_ins.additional_str(),
  #   )
  #
  #   with open(
  #       os.path.join(os.path.dirname(__file__), "op_code_generators",
  #                    "{}.py".format(schema.name)), "w") as f:
  #     f.write(gened_op_code_generator)
  #   pass
  pass

gen_code()


from onnx_pytorch.code_gen_templates.temp_formatter import TempFormatter
from onnx_pytorch.code_trans import CodeTrans


class BatchNormalizationCodeTrans(CodeTrans):

  def __init__(self, schema):
    super(BatchNormalizationCodeTrans, self).__init__(schema)

  def get_torch_input_num(self):
    return 1

  def get_nn_name_str(self):
    return "\"BatchNorm{}d\".format(len(value_infos[node.input[0]].type.tensor_type.shape.dim) - 2)"

  def init_str(self):
    init_str = f'''
    \'\'\'self.{{node_name}} = nn.{{nn_name}}(**{{params}})
    self.{{node_name}}.weight.data = self.__variables[\"{{weight_name}}\"]
    if {{bias}}:
      self.{{node_name}}.bias.data = self.__variables[\"{{bias_name}}\"]
\'\'\'.format(
    nn_name={self.get_nn_name_str()},
    weight_name=node.input[1],
    bias_name="Undefined" if len(node.input) == 2 else node.input[2],
    node_name=node.name,
    bias=len(node.input) > 2,
    params=params_str,)'''
    return init_str

  def trans_input(self):
    pass

  def trans_attr(self):
    self.add_attr("epsilon", "eps")
    self.add_attr("momentum", "momentum")
    self.add_attr(
        "", "num_features",
        "{pytorch_attr}=onnx.numpy_helper.to_array(initializers[node.input[1]]).shape[0]"
    )
